'use strict'

Object.defineProperty(exports, '__esModule', { value: true })

function _interopDefault(ex) {
  return ex && typeof ex === 'object' && 'default' in ex ? ex['default'] : ex
}

var path = require('path')
var Vue = _interopDefault(require('vue'))
var compositionApi = require('@vue/composition-api')

function getValue(value) {
  if (typeof value === 'function') return value()
  return value
}
let ssrContext
const refs = []
function injectRefs() {
  if (!process.server || !ssrContext) return
  if (!ssrContext.nuxt.ssrRefs) ssrContext.nuxt.ssrRefs = {}
  refs.forEach(([key, ref]) => {
    ssrContext.nuxt.ssrRefs[key] = ref.value
  })
}
const ssrRef = (value, key) => {
  var _a
  const val = compositionApi.ref(getValue(value))
  const vm = compositionApi.getCurrentInstance()
  if (!key)
    throw new Error(
      "You must provide a key. You can have it generated automatically by adding 'nuxt-composition-api/babel' to your Babel plugins."
    )
  if (!ssrContext) {
    ssrContext = ssrContext || vm.$ssrContext
    compositionApi.onServerPrefetch(injectRefs)
  }
  if (process.client) {
    const nuxtState = window.__NUXT__
    val.value =
      (_a = (nuxtState.ssrRefs || {})[key]) !== null && _a !== void 0
        ? _a
        : getValue(value)
  } else {
    refs.push([key, val])
  }
  return val
}
const onServerPrefetch = callback => {
  compositionApi.onServerPrefetch(async () => {
    await callback()
    injectRefs()
  })
}

function normalizeError(err) {
  let message
  if (!(err.message || typeof err === 'string')) {
    try {
      message = JSON.stringify(err, null, 2)
    } catch (e) {
      message = `[${err.constructor.name}]`
    }
  } else {
    message = err.message || err
  }
  return {
    ...err,
    message,
    statusCode:
      err.statusCode ||
      err.status ||
      (err.response && err.response.status) ||
      500,
  }
}
const fetches = new WeakMap()
const isSsrHydration = vm => {
  var _a, _b, _c
  return (_c =
    (_b = (_a = vm.$vnode) === null || _a === void 0 ? void 0 : _a.elm) ===
      null || _b === void 0
      ? void 0
      : _b.dataset) === null || _c === void 0
    ? void 0
    : _c.fetchKey
}
const nuxtState = process.client && window.__NUXT__
function registerCallback(vm, callback) {
  const callbacks = fetches.get(vm) || []
  fetches.set(vm, [...callbacks, callback])
}
async function callFetches() {
  const fetchesToCall = fetches.get(this)
  if (!fetchesToCall) return
  this.$nuxt.nbFetching++
  this.$fetchState.pending = true
  this.$fetchState.error = null
  this._hydrated = false
  let error = null
  const startTime = Date.now()
  try {
    await Promise.all(fetchesToCall.map(fetch => fetch(this)))
  } catch (err) {
    error = normalizeError(err)
  }
  const delayLeft = (this._fetchDelay || 0) - (Date.now() - startTime)
  if (delayLeft > 0) {
    await new Promise(resolve => setTimeout(resolve, delayLeft))
  }
  this.$fetchState.error = error
  this.$fetchState.pending = false
  this.$fetchState.timestamp = Date.now()
  this.$nextTick(() => this.$nuxt.nbFetching--)
}
async function serverPrefetch(vm) {
  if (!vm._fetchOnServer) {
    return
  }
  // Call and await on $fetch
  vm.$fetchState =
    vm.$fetchState ||
    Vue.observable({
      error: null,
      pending: false,
      timestamp: 0,
    })
  try {
    await callFetches.call(vm)
  } catch (err) {
    vm.$fetchState.error = normalizeError(err)
  }
  vm.$fetchState.pending = false
  // Define an ssrKey for hydration
  vm._fetchKey = vm.$ssrContext.nuxt.fetch.length
  // Add data-fetch-key on parent element of Component
  if (!vm.$vnode.data) vm.$vnode.data = {}
  const attrs = (vm.$vnode.data.attrs = vm.$vnode.data.attrs || {})
  attrs['data-fetch-key'] = vm._fetchKey
  // Add to ssrContext for window.__NUXT__.fetch
  vm.$ssrContext.nuxt.fetch.push(
    vm.$fetchState.error
      ? { _error: vm.$fetchState.error }
      : JSON.parse(JSON.stringify(vm._data))
  )
}
const useFetch = callback => {
  var _a
  const vm = compositionApi.getCurrentInstance()
  if (!vm) throw new Error('This must be called within a setup function.')
  vm.$fetchState =
    vm.$fetchState ||
    Vue.observable({
      error: null,
      pending: false,
      timestamp: 0,
    })
  vm.$fetch = callFetches.bind(vm)
  registerCallback(vm, callback)
  if (typeof vm.$options.fetchOnServer === 'function') {
    vm._fetchOnServer = vm.$options.fetchOnServer.call(vm) !== false
  } else {
    vm._fetchOnServer = vm.$options.fetchOnServer !== false
  }
  onServerPrefetch(() => serverPrefetch(vm))
  compositionApi.onBeforeMount(() => {
    if (!vm._hydrated) {
      return callFetches.call(vm)
    }
  })
  if (process.server || !isSsrHydration(vm)) return
  // Hydrate component
  vm._hydrated = true
  vm._fetchKey = +((_a = vm.$vnode.elm) === null || _a === void 0
    ? void 0
    : _a.dataset.fetchKey)
  const data = nuxtState.fetch[vm._fetchKey]
  // If fetch error
  if (data && data._error) {
    vm.$fetchState.error = data._error
    return
  }
  compositionApi.onBeforeMount(() => {
    // Merge data
    for (const key in data) {
      try {
        Vue.set(vm, key, data[key])
      } catch (e) {
        if (process.env.NODE_ENV === 'development')
          // eslint-disable-next-line
          console.warn(`Could not hydrate ${key}.`)
      }
    }
  })
}

const withContext = callback => {
  const vm = compositionApi.getCurrentInstance()
  if (!vm) throw new Error('This must be called within a setup function.')
  callback(vm.$nuxt.context)
}

const compositionApiModule = function () {
  const libRoot = path.resolve(__dirname, '..')
  const { dst } = this.addTemplate({
    src: path.resolve(libRoot, 'lib', 'plugin.js'),
    fileName: path.join('composition-api', 'plugin.js'),
    options: {},
  })
  this.options.build = this.options.build || {}
  this.options.build.babel = this.options.build.babel || {}
  this.options.build.babel.plugins = this.options.build.babel.plugins || []
  this.options.build.babel.plugins.push(path.join(__dirname, 'babel'))
  this.options.plugins = this.options.plugins || []
  this.options.plugins.push(path.resolve(this.options.buildDir || '', dst))
}
// eslint-disable-next-line
const meta = require('../package.json')

Object.defineProperty(exports, 'computed', {
  enumerable: true,
  get: function () {
    return compositionApi.computed
  },
})
Object.defineProperty(exports, 'createComponent', {
  enumerable: true,
  get: function () {
    return compositionApi.createComponent
  },
})
Object.defineProperty(exports, 'createElement', {
  enumerable: true,
  get: function () {
    return compositionApi.createElement
  },
})
Object.defineProperty(exports, 'defineComponent', {
  enumerable: true,
  get: function () {
    return compositionApi.defineComponent
  },
})
Object.defineProperty(exports, 'getCurrentInstance', {
  enumerable: true,
  get: function () {
    return compositionApi.getCurrentInstance
  },
})
Object.defineProperty(exports, 'inject', {
  enumerable: true,
  get: function () {
    return compositionApi.inject
  },
})
Object.defineProperty(exports, 'isRef', {
  enumerable: true,
  get: function () {
    return compositionApi.isRef
  },
})
Object.defineProperty(exports, 'onActivated', {
  enumerable: true,
  get: function () {
    return compositionApi.onActivated
  },
})
Object.defineProperty(exports, 'onBeforeMount', {
  enumerable: true,
  get: function () {
    return compositionApi.onBeforeMount
  },
})
Object.defineProperty(exports, 'onBeforeUnmount', {
  enumerable: true,
  get: function () {
    return compositionApi.onBeforeUnmount
  },
})
Object.defineProperty(exports, 'onBeforeUpdate', {
  enumerable: true,
  get: function () {
    return compositionApi.onBeforeUpdate
  },
})
Object.defineProperty(exports, 'onDeactivated', {
  enumerable: true,
  get: function () {
    return compositionApi.onDeactivated
  },
})
Object.defineProperty(exports, 'onErrorCaptured', {
  enumerable: true,
  get: function () {
    return compositionApi.onErrorCaptured
  },
})
Object.defineProperty(exports, 'onMounted', {
  enumerable: true,
  get: function () {
    return compositionApi.onMounted
  },
})
Object.defineProperty(exports, 'onUnmounted', {
  enumerable: true,
  get: function () {
    return compositionApi.onUnmounted
  },
})
Object.defineProperty(exports, 'onUpdated', {
  enumerable: true,
  get: function () {
    return compositionApi.onUpdated
  },
})
Object.defineProperty(exports, 'provide', {
  enumerable: true,
  get: function () {
    return compositionApi.provide
  },
})
Object.defineProperty(exports, 'reactive', {
  enumerable: true,
  get: function () {
    return compositionApi.reactive
  },
})
Object.defineProperty(exports, 'ref', {
  enumerable: true,
  get: function () {
    return compositionApi.ref
  },
})
Object.defineProperty(exports, 'set', {
  enumerable: true,
  get: function () {
    return compositionApi.set
  },
})
Object.defineProperty(exports, 'toRefs', {
  enumerable: true,
  get: function () {
    return compositionApi.toRefs
  },
})
Object.defineProperty(exports, 'watch', {
  enumerable: true,
  get: function () {
    return compositionApi.watch
  },
})
Object.defineProperty(exports, 'watchEffect', {
  enumerable: true,
  get: function () {
    return compositionApi.watchEffect
  },
})
exports.default = compositionApiModule
exports.meta = meta
exports.onServerPrefetch = onServerPrefetch
exports.ssrRef = ssrRef
exports.useFetch = useFetch
exports.withContext = withContext
